--// vim:ts=4:sw=4:noet
--// ignore.lua -- makes possible to (un)ignore someone from the chat

if not ignoretable then
	ignoretable = {}
	ignoretable.settingsfile = "scripts/ignoretable.txt"
	ignoretable.users = {}
	ignoretable.permament = 1
	ignoretable.private = 0
end

if not ignorefunctions then
	ignorefunctions = {}
end

function ignorefunctions.SaveSettings()
	pickle.store(ignoretable.settingsfile, { ignoretable = ignoretable })
end

function ignorefunctions.ClearUsers()
	-- clears all users from the table
	while table.getn(ignoretable.users) > 0 do
		table.remove(ignoretable.users)
	end
	ignorefunctions.SaveSettings()
end

function ignorefunctions.LoadSettings()
	local o = io.open( ignoretable.settingsfile, "r" )
	if o then
		dofile( ignoretable.settingsfile )
		o:close()
	end
	-- "ignoretable.permament" decides whether to store the ignored messages after restarting the program
	if (ignoretable.permament == 0) then
		ignorefunctions.ClearUsers()
	end
end

dofile("scripts/libsimplepickle.lua")
ignorefunctions.LoadSettings()

function ignorefunctions.OnOff(variable)
	local ret = "on"
	if (variable == 0) or (variable == nil) then
		ret = "off"
	end
	return ret
end

function ignorefunctions.IsIgnored(nick)
	local userignored = false
	for k in ignoretable.users do
		if (ignoretable.users[k] == nick) then
			userignored = true
			break
		end
	end
	return userignored
end

function ignorefunctions.ListIgnore(hub)
	local listofusers=""
	local first = true
	for k in ignoretable.users do
		if first == true then
			first = false
		else
			listofusers = listofusers..", "
		end
		listofusers=listofusers..ignoretable.users[k]
	end
	hub:injectChat("*** Currently ignored users: "..listofusers)
end

function ignorefunctions.AddUser(nick)
	table.insert(ignoretable.users, nick)
	ignorefunctions.SaveSettings()
end

function ignorefunctions.RmUser(nick)
	local managed = false
	if ignorefunctions.IsIgnored(nick) then
		for k in ignoretable.users do
			if (ignoretable.users[k] == nick) then
				table.remove(ignoretable.users, k)
				managed = true
				ignorefunctions.SaveSettings()
			end
		end
	else
		-- he's not ignored
		managed = false
	end
	return managed
end

-- This is similar to formatting.lua's, but needed a little change
function ignorefunctions.tokenize( str )
	local ret = {}
	string.gsub( str, "([^/%s]+)", function( s ) table.insert( ret, s ) end )
	return ret
end

dcpp:setListener("connected", "usercommands_ignore", function(hub)
	if (hub:isADC()) then
		-- what about adc hub?
		-- DC():PrintDebug("ADC hub..")
	else
		-- DC():PrintDebug("NMDC hub..")
		-- and what about this? :)
--		hub:injectChat("$UserCommand 1 2 Ignore$<%[mynick]> /ignore %[nick]&#124;|")
--		hub:injectChat("$UserCommand 1 2 Unignore$<%[mynick]> /unignore %[nick]&#124;|")
	end
end)

dcpp:setListener("ownChatOut", "entered_ignore", function(hub, message, ret)
	if string.sub( message, 1, 1) ~= "/" then
		return nil
	end
	local params = ignorefunctions.tokenize( message )
	if params[1] == "help" then
		hub:injectChat( "*** (ignore.lua) /ignore <user>, /unignore <user>, /listignore, /ignorepm <on/off> (curr: ".. ignorefunctions.OnOff(ignoretable.private).."), /permament <on/off> (curr: ".. ignorefunctions.OnOff(ignoretable.permament) .."), /purgelist" )
		return nil
	elseif params[1] == "ignore" then
		if params[2] then
			ignorefunctions.AddUser(params[2])
			hub:injectChat("*** " .. params[2] .. " ignored")
		else
			hub:injectChat("*** Usage: /ignore <nick>")
		end
		return 1
	elseif params[1] == "unignore" then
		if params[2] then
			if ignorefunctions.RmUser(params[2]) then
				hub:injectChat("*** " .. params[2] .. " unignored")
			else
				hub:injectChat("*** " .. params[2] .. " couldn't removed, please check /listignore")
			end	
		else
			hub:injectChat("*** Usage: /unignore <nick>")
		end
		return 1
	elseif params[1] == "listignore" then
		ignorefunctions.ListIgnore(hub)
		return 1
	elseif params[1] == "ignorepm" then
		if params[2] == "on" then
			ignoretable.private = 1
			hub:injectChat("*** Private messages are ignored for the selected users. CAUTION: Take your own responsibility if you miss an important message!")
			ignorefunctions.SaveSettings()
		elseif params[2] == "off" then
			ignoretable.private = 0
			hub:injectChat("*** Private messages no more ignored")
			ignorefunctions.SaveSettings()		
		else
			hub:injectChat("*** Usage: /ignorepm <on/off>. Currently PM ignoring is turned "..ignorefunctions.OnOff(ignoretable.private)..".")
		end
		return 1
	elseif params[1] == "permament" then
		if params[2] == "on" then
			ignoretable.permament = 1
			hub:injectChat("*** Ignored users will be keeped after program restart")
			ignorefunctions.SaveSettings()
		elseif params[2] == "off" then
			ignoretable.permament = 0
			hub:injectChat("*** Ignored users won't be keeped after program restart")
			ignorefunctions.SaveSettings()
		else
			hub:injectChat("*** Usage: /permament <on/off>. Currently ignorelist saving is turned "..ignorefunctions.OnOff(ignoretable.permament)..".")
		end
		return 1
	elseif params[1] == "purgelist" then
		ignorefunctions.ClearUsers()
		hub:injectChat("*** Ignored users' list cleared")
		return 1
	end
end)

dcpp:setListener("chat", "chat_ignore", function( hub, user, text )
	local nick = user:getNick()
	if ignorefunctions.IsIgnored(nick) then
		DC():PrintDebug( "Ignored message from: " .. nick.. " in main chat.")
		return 1
	end
	return nil
end)

dcpp:setListener("pm", "pm_ignore", function( hub, user, message )
	local nick = user:getNick()
	if ignorefunctions.IsIgnored(nick) and (ignoretable.private == 1) then
		DC():PrintDebug( "Ignored message from: " .. nick .. " in private chat")
		return 1
	end
	return nil
end)

DC():PrintDebug( "  ** Loaded ignore.lua **" )
